#include <owl/applicat.h>
#include <owl/dialog.h>
#include <owl/framewin.h>

#include "clock.h"
#include "dlgclock.rc"

#define WM_TRAYICON_MSG  WM_USER+100

//
// class TClockDlg
// ~~~~~ ~~~~~~~~~
//
class TClockDlg : public TDialog {
  public:
    TClockDlg(TWindow* parent);
};

TClockDlg::TClockDlg(TWindow* parent)
:
 TDialog(parent, IDD_DIALOG)
{
  new TClock(this, IDC_CLOCKCTL);
}

//
// class TClockFrm
// ~~~~~ ~~~~~~~~~
//
class TClockFrm : public TFrameWindow {
  public:
    TClockFrm(TWindow* parent, const char* title = 0, TWindow* clientWnd = 0,
              bool shrinkToClient = false, TModule* module = 0);

    void SetupWindow  ();
    void CleanupWindow();

  protected:
    void EvSysCommand(uint cmdType, TPoint& point);

    LRESULT EvTrayIconMsg(WPARAM wParam, LPARAM lParam);

  private:
    NOTIFYICONDATA* tnd;

  DECLARE_RESPONSE_TABLE(TClockFrm);
};

DEFINE_RESPONSE_TABLE1(TClockFrm, TFrameWindow)
  EV_WM_SYSCOMMAND,
  EV_MESSAGE(WM_TRAYICON_MSG, EvTrayIconMsg),
END_RESPONSE_TABLE;

TClockFrm::TClockFrm(TWindow* parent, const char* title, TWindow* clientWnd,
                     bool shrinkToClient, TModule* module)
:
 TFrameWindow(parent, title, clientWnd, shrinkToClient, module),
 tnd(0)
{
}

void
TClockFrm::SetupWindow()
{
  TFrameWindow::SetupWindow();

  tnd = new NOTIFYICONDATA;

  tnd->cbSize           = sizeof(NOTIFYICONDATA);
  tnd->hWnd             = *this;
  tnd->uID              = IDI_ICON;
  tnd->uFlags           = NIF_ICON | NIF_MESSAGE | NIF_TIP;
  tnd->uCallbackMessage = WM_TRAYICON_MSG;
  tnd->hIcon            = ::Module->LoadIcon(MAKEINTRESOURCE(IDI_ICON));

  strcpy(tnd->szTip, "Watch Your Time");

  Shell_NotifyIcon(NIM_ADD, tnd);
}

void
TClockFrm::CleanupWindow()
{
  Shell_NotifyIcon(NIM_DELETE, tnd);

  delete tnd;

  TFrameWindow::CleanupWindow();
}

void
TClockFrm::EvSysCommand(uint cmdType, TPoint& point)
{
  TFrameWindow::EvSysCommand(cmdType, point);

  if (cmdType == SC_MINIMIZE)
    ShowWindow(SW_HIDE);
}

LRESULT
TClockFrm::EvTrayIconMsg(WPARAM /* wParam */, LPARAM lParam)
{
  if (lParam == WM_LBUTTONDOWN) {
    if (!IsWindowVisible()) {
      ShowWindow(SW_SHOWMINNOACTIVE);
      ShowWindow(SW_RESTORE);
    }
  }

  return TRUE;
}

//
// class TClockApp
// ~~~~~ ~~~~~~~~~
//
class TClockApp : public TApplication {
  public:
    TClockApp(char* name);

    void InitMainWindow();
};

TClockApp::TClockApp(char* name)
:
 TApplication(name)
{
}

void
TClockApp::InitMainWindow()
{
  TClockFrm* frame = new TClockFrm(0, "Clock", new TClockDlg(0), true);
  frame->ModifyStyle(WS_MAXIMIZEBOX | WS_SIZEBOX, 0);
  frame->SetIconSm(::Module, IDI_ICON);

  SetMainWindow(frame);
}

int
OwlMain(int /* argc */, char* /* argv */ [])
{
  return TClockApp("Clock").Run();
}
